"""An inbound email handler for Django."""

__title__ = 'django-inbound-email'
__version__ = '0.7'
__author__ = 'YunoJuno Ltd'
__license__ = 'MIT'
__copyright__ = 'Copyright 2014 YunoJuno'
__description__ = 'A Django app for receiving inbound emails.'
